// C Source File
// Created 7/19/2002; 6:02:05 PM

/*---------------------------------
		   Arena 3D Level Editor
	      By Malcolm Smith
	      Version v0.6
----------------------------------*/

#define USE_TI89		// Compile for TI-89
#define USE_TI92PLUS		// Compile for TI-92 Plus
#define USE_V200		// Compile for V200

#define OPTIMIZE_ROM_CALLS	// Use ROM Call Optimization

#define MIN_AMS 100		// Compile for AMS 1.00 or higher

#define SAVE_SCREEN		// Save/Restore LCD Contents

#include <tigcclib.h>		// Include All Header Files
#include "extgraph.h"
#include "i_a3dgfx1.h"
#include "edit_draw.h"
#include "edit_globals.h"
#include "lvlfind_.h"
#include "levmenu_.h"
#include "keyscan_edit.h"
#include "save.h"


// macro returns the address of tile nr of plane p from array ball_gfx
#define GFXTILE(p,nr) &(data[p][TILEHEIGHT_DATA*nr])
#define _findme(a,b) (map_width*(b) + (a))

#ifndef FileConstants
#define FileConstants
enum FileConstants
{ HOWMANY = 50, MAXLEN = 9 };
#endif


DEFINE_INT_HANDLER (count_int)
{
  ++count;
}

void
delay (unsigned short del)	//in 55 ms blocks
{
  unsigned short start = count;
  while (count - start < del)	/* nothing */
    ;
}

void
delay1 (unsigned short del)	//in 2.857 ms blocks
{
  unsigned short start = count1;
  while (count1 - start < del)	/* nothing */
    ;
}


char curfolder[9];

short
MakeNewLevel (void)
{
  char tempstr[5];
  clrscr ();
  FontSetSys (F_6x8);
  printf ("Game Mode?\n0=DM,2=CTF\n");
  InputNum (tempstr, 4);
  gametype = atoi (tempstr);
  if (gametype != 0 && gametype != 2)
    gametype = 0;
  if (gametype == 0)
    tiles_offset = 5;
  else
    tiles_offset = 0;
  printf
    ("\nNote that using map\ndimensions that are powers\nof 2 increases in-game\nspeed.\n");
  printf ("\nMap Width [x]:\n");
  InputNum (tempstr, 4);
  map_width = atoi (tempstr);
  printf ("\nMap Height [y]:\n");
  InputNum (tempstr, 4);
  map_height = atoi (tempstr);
  if (map_width == 0 || map_height == 0)
    return 0;

  vidmem = (char *) malloc (sizeof (char) * map_width * map_height);
//Loads empty level
  memset (vidmem, 0, map_width * map_height);


  FontSetSys (F_6x8);
  printf ("\nOpening empty map...");
  delay (14);
  return 1;
}

char *gamefolder;
// Main Function
void
_main (void)
{

  unsigned short select_tile[] =	//For XORing as an inverse
  { 0xffff, 0xc003, 0xa005, 0x9009, 0x8811, 0x8421, 0x8241, 0x8181,
    0x8181, 0x8241, 0x8421, 0x8811, 0x9009, 0xa005, 0xc003, 0xffff
  };
  unsigned short empty_tile[] =	//For clearing out a space for the balls
  { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
    0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000
  };



  short pos_y = 0;
  short pos_x = 0;





  char vidmem_cur = 0;
//short error=0;
  short number = 0;
  short total_maps = 0;
  short key;
  unsigned short count_current;
  short inc;

  char filestr[9] = { 0 };
  
  void *aplane0, *aplane1, *hplane0, *hplane1;
  char buffer[HOWMANY * MAXLEN];
  SYM_ENTRY *symptr = NULL;
  unsigned char *filedata;

  MAP_HEADER cur = { 0, 0, 0, 0, {0}, {0}, -1 };

  SaveInt1 = NULL;
  SaveInt5 = NULL;
  screen0 = NULL;
  screen1 = NULL;
  vidmem = NULL;



  SaveInt1 = GetIntVec (AUTO_INT_1);
  SaveInt5 = GetIntVec (AUTO_INT_5);
  SetIntVec (AUTO_INT_5, count_int);

  ClrScr ();
  FontSetSys (F_6x8);
  DrawStr (0, 10, "Welcome to the", A_NORMAL);
  DrawStr (0, 21, "Arena 3D Level Editor!", A_NORMAL);
  DrawStr (0, 32, "By Malcolm Smith", A_NORMAL);
  DrawStr (0, 50, "89 (92+/V200) Keys...", A_NORMAL);
  DrawStr (0, 58, "Arrows      -Move Around", A_NORMAL);
  DrawStr (0, 66, "2nd (Shift) -Texture Down", A_NORMAL);
  DrawStr (0, 74, "Shift (Z)   -Texture Up", A_NORMAL);
  DrawStr (0, 82, "Diamond     -Place Texture", A_NORMAL);

  key = ngetchx ();
  if (key == KEY_ESC)
    goto leadout;

  ClrScr ();
  FontSetSys (F_8x10);
  DrawStr (2, 2, "Would you like to", A_NORMAL);
  DrawStr (2, 13, "open a map?", A_NORMAL);
  DrawStr (2, 24, "Yes-Enter", A_NORMAL);
  DrawStr (2, 35, "No-Esc", A_NORMAL);


  key = ngetchx ();

  if (key == KEY_ENTER)
    {
      //Load Level
      number = findAllFilesSet (buffer, "alev", HOWMANY);
      if (!number)
	number = -1;
      if (number != -1)		//OK, there are levels on the calc
	{
	  ClrScr ();
	  number = domenu (35, 28, buffer, number, 0);
	}

      if (number != -1)		//OK, a level was selected
	{
	  strcpy (filestr, buffer + number * MAXLEN);
	  symptr = DerefSym (SymFind (SYMSTR (filestr)));
	  if (symptr)
	    {
	      filedata = HLock (symptr->handle);
	      filedata += 2;
//Get the total number of maps in the file
	      total_maps = *(short *) filedata;

//next comes a list of MAP_HEADERS
	      MAP_HEADER *header = (MAP_HEADER *) (filedata + 2);
//next comes the actual tilemaps
	      unsigned char *map_data =
		(unsigned char *) (filedata + 2 +
				   sizeof (MAP_HEADER) * total_maps);

//adding the offset for the ith header to the start of the tilemaps will get a pointer
//to the start of the tilemap we want
	      unsigned char *get_tile_map (unsigned short i)
	      {
		return map_data + header[i].offset;
	      }
	      inc = 0;
	      FontSetSys (F_6x8);


	      pushkey (KEY_CLEAR);
	      while ((key = ngetchx ()) != KEY_ENTER)
		{
		  ClrScr ();
		  DrawStr (1, 90, "<<", A_NORMAL);
		  DrawStr (146, 90, ">>", A_NORMAL);

		  if (key == KEY_LEFT)
		    inc--;
		  else if (key == KEY_RIGHT)
		    inc++;

		  if (inc == -1)
		    inc = total_maps - 1;
		  if (inc == total_maps)
		    inc = 0;

		  DrawStr (16, 32, (header + inc)->title, A_NORMAL);
		  DrawStr (16, 42, (header + inc)->author, A_NORMAL);
		  printf_xy (40, 60, "Dims: [%i,%i] %i",
			     (header + inc)->xsize, (header + inc)->ysize,
			     (header + inc)->offset);
		  printf_xy (40, 80, "INDEX #%i  ", inc);
		}
	      map_width = (header + inc)->xsize;
	      map_height = (header + inc)->ysize;
	      strcpy (cur.title, (header + inc)->title);
	      strcpy (cur.author, (header + inc)->author);
	      cur.background = (header + inc)->background;
	      vidmem =
		(char *) malloc (sizeof (char) * map_width * map_height);
	      if (vidmem == NULL)
		goto leadout;

	      memcpy (vidmem, get_tile_map (inc),
		      sizeof (char) * map_width * map_height);

	    }

	}
      if (number == -1)
	{
	  if (!MakeNewLevel ())
	    goto leadout;
	}

    }
  else				// if (key == KEY_ESC)
    {
      clrscr ();
      if (!MakeNewLevel ())
	goto leadout;
    }




  xmax = (map_width < X_MAX) ? map_width : X_MAX;
  ymax = (map_height < Y_MAX) ? map_height : Y_MAX;

//Redirects interrupts to nothing for _rowread
  SetIntVec (AUTO_INT_1, DUMMY_HANDLER);

  screen0 = (char *) malloc (3840);
  screen1 = (char *) malloc (3840);
  if ((screen0 == NULL) || ((screen1 == NULL)))
    goto leadout;
  memset (screen0, 0, 3840);
  memset (screen1, 0, 3840);
  if (!GrayOn ())
    goto leadout;


  keyscan_init ();

  aplane0 = GrayGetPlane (0);
  aplane1 = GrayGetPlane (1);
  hplane0 = screen0;
  hplane1 = screen1;


  keyscan ();


  while (!keystat.escape)
    {

      count_current = count;
      keyscan ();
      draw_edit_balls ();

      if (vidmem_cur)
	{
	  GraySprite16_MASK (pos_x * 16, pos_y * 16, 16,
			     GFXTILE (0, (vidmem_cur + 4)), GFXTILE (1,
								     (vidmem_cur
								      + 4)),
			     empty_tile, empty_tile, hplane0, hplane1);
	}
      else
	{
	  GraySprite16_XOR (pos_x * 16, pos_y * 16, 16,
			    select_tile, select_tile, hplane0, hplane1);
	}

      if (keystat.left)
	{
	  pos_x--;
	  if (pos_x == -1)
	    {
	      if (screen_x > 0)
		{
		  screen_x--;
		  pos_x++;
		}
	      else
		pos_x = 0;
	    }
	}
      if (keystat.right)
	{
	  pos_x++;
	  if (pos_x == xmax)
	    {
	      if (screen_x < map_width - xmax)
		{
		  screen_x++;
		  pos_x--;
		}
	      else
		pos_x = xmax - 1;
	    }
	}
      if (keystat.up)
	{
	  pos_y--;
	  if (pos_y == -1)
	    {
	      if (screen_y > 0)
		{
		  pos_y++;
		  screen_y--;
		}
	      else
		pos_y = 0;
	    }
	}
      if (keystat.down)
	{
	  pos_y++;
	  if (pos_y == ymax)
	    {
	      if (screen_y < map_height - ymax)
		{
		  screen_y++;
		  pos_y--;
		}
	      else
		pos_y = ymax - 1;
	    }
	}
      if (keystat.diamond)
	{
	  vidmem[_findme (pos_x + screen_x, pos_y + screen_y)] = vidmem_cur;
	}
      if (keystat.shift)
	{
	  if (vidmem_cur < (TILENUMBER_DATA - 6))
	    vidmem_cur++;
	}
      if (keystat.second)
	{
	  if (vidmem_cur > (gametype ? -4 : 0))
	    vidmem_cur--;
	}
      FastCopyScreen (hplane0, aplane0);
      FastCopyScreen (hplane1, aplane1);
      if (keystat.up || keystat.down || keystat.left || keystat.right
	  || keystat.diamond)
	while ((count_current + 4) > count);	// + 5
      while (keystat.second || keystat.shift)
	keyscan ();
    }


  if (GrayCheckRunning ())
    GrayOff ();
  if (screen0 != NULL)
    {
      free (screen0);
      screen0 = NULL;
    }
  if (screen1 != NULL)
    {
      free (screen1);
      screen1 = NULL;
    }
  if (SaveInt1 != NULL)
    SetIntVec (AUTO_INT_1, SaveInt1);
  if (SaveInt5 != NULL)
    SetIntVec (AUTO_INT_5, SaveInt5);
  FontSetSys (F_8x10);
archived:
  clrscr ();
  printf ("Save your map? \nEnter=Yes\n");
  GKeyFlush ();
  key = ngetchx ();

  if (key == KEY_ENTER)
    {
      short flag = 0;
      if (filestr[0] && cur.title[0] && cur.author[0] && cur.background != -1)
	{
	  GKeyFlush ();
	  printf ("\nKeep Same Info?\nEnter=Yes\n");
	  key = ngetchx ();
	  if (key == KEY_ENTER)
	    flag = 1;
	}

      if (!flag)
	{
	  FontSetSys (F_6x8);
	  printf ("Filename:\n");
	  InputStr (filestr, 8);
	}

      if (filestr[0] != 0)
	{			//If a name was typed in
	  if (checkarchived (filestr))
	    {
	      FontSetSys (F_6x8);
	      printf ("\n\nThat level is archived.\n");
	      ngetchx ();
	      FontSetSys (F_8x10);
	      goto archived;
	    }
	  if (!flag)
	    {
	      printf ("\nTitle:\n");
	      InputInfo (cur.title, 26);
	      printf ("\nAuthor:\n");
	      InputInfo (cur.author, 26);
	    }


	  cur.xsize = map_width;
	  cur.ysize = map_height;
	  cur.gametype = gametype;

	  inc = 0;
	  ClrScr ();
	  FontSetSys (F_6x8);
	  DrawStr (1, 90, "<<", A_NORMAL);
	  DrawStr (146, 90, ">>", A_NORMAL);
	  pushkey (KEY_CLEAR);

	  while ((key = ngetchx ()) != KEY_ENTER)
	    {
	      char spaces[27];
	      memset (spaces, ' ', 27);
	      if (key == KEY_LEFT)
		inc--;
	      else if (key == KEY_RIGHT)
		inc++;

	      if (inc == -1)
		inc = MAX_BGS - 1;
	      if (inc == MAX_BGS)
		inc = 0;
	      DrawStr (0, 32, spaces, A_REPLACE);
	      DrawStr (0, 32, Backgrounds[inc], A_NORMAL);
	      printf_xy (40, 80, "INDEX #%i", inc);

	    }
	  cur.background = inc;
	  cur.offset = 0;
	  {
	    clrscr ();
	    FontSetSys (F_6x8);
	    printf ("\nSUMMARY:\n");
	    printf ("\nFilename: %s", filestr);
	    printf ("\nTitle:\n%s", cur.title);
	    printf ("\nAuthor:\n%s", cur.author);
	    printf ("\nBackground:\n%s", Backgrounds[inc]);
	    ngetchx ();
	  }
	  savefile (&cur, filestr);
	}


    }

leadout:
  if (GrayCheckRunning ())
    GrayOff ();
  if (symptr)
    HeapUnlock (symptr->handle);
  if (screen0 != NULL)
    {
      free (screen0);
      screen0 = NULL;
    }
  if (screen1 != NULL)
    {
      free (screen1);
      screen1 = NULL;
    }
  if (vidmem != NULL)
    {
      free (vidmem);
      vidmem = NULL;
    }
  if (SaveInt1 != NULL)
    SetIntVec (AUTO_INT_1, SaveInt1);
  if (SaveInt5 != NULL)
    SetIntVec (AUTO_INT_5, SaveInt5);
}
